//----------------------------------------------------------------------------
// utilities
//----------------------------------------------------------------------------

var objShell = new ActiveXObject("WScript.Shell");
var objFS = new ActiveXObject("Scripting.FileSystemObject");
// var objConn = new ActiveXObject("ADODB.Connection");

var objXmlHttp;
try {
	objXmlHttp = new ActiveXObject('Msxml2.XMLHTTP');
}
catch (e) {
	objXmlHttp = new ActiveXObject('Microsoft.XMLHTTP');
}

var adTypeBinary = 1;
var adTypeText = 2;
var adSaveCreateOverWrite = 2;

var ForReading = 1;
var ForWriting = 2;
var ForAppending = 8;

var Log_Error = 0;
var Log_Trace = 1;
var Log_Debug = 2;
var LogPrefix = ["[ERROR]", "[TRACE]", "[DEBUG]"];


///////////////////////////////////////////////////////////////////////
// Utils

function Utils()
{
	if (typeof(WScript) != "undefined") {
		this.appDir = objFS.GetParentFolderName(WScript.ScriptFullName);
	}
	else if (typeof(document) != "undefined") {
		this.appDir = unescape(objFS.GetParentFolderName(document.location.pathname));
	}
	else {
		this.appDir = objShell.CurrentDirectory;
	}
	this.userDir = objFS.BuildPath(objShell.SpecialFolders.Item("MyDocuments"), "ASConsole");
	if (! objFS.FolderExists(this.userDir)) {
		objFS.CreateFolder(this.userDir);
	}
	this.logLevel = Log_Error;
	this.sharedVariables = new ActiveXObject("Scripting.Dictionary");
	this.logPath = null;
	this.itemSeparator = ", "
	this.messages = messageTable.Item("en");
	this.language = "en";
	this.objError = null;
	this.postAction = null;
	this.scRubyScript = null;
	this.jsBrowserAttached = false;
	this.jsBrowserAttachedPrev = false;
	this.jsBrowserTitle = null;
}

// for debug (temporary method)
Utils.prototype.typeCheck = function(v)
{
	utils.println(typeof(v));
	utils.println(__tostr(v));
	utils.println((v == null) ? "v is null" : "v is not null");
	utils.println((typeof(v) == "undefined") ? "v is undefined" : "v is defined");
}

Utils.prototype.setSeparator = function(s)
{
	this.itemSeparator = __tostr(s);
}

Utils.prototype.getSeparator = function()
{
	return this.itemSeparator;
}

Utils.prototype.getAppDir = function()
{
	return this.appDir;
}

Utils.prototype.getUserDir = function()
{
	return this.userDir;
}

Utils.prototype.getLanguage = function()
{
	return this.language;
}

Utils.prototype.getScriptManager = function()
{
	return scriptManager;
}

Utils.prototype.setVariable = function(name, value)
{
	if (name == null) {
		return;
	}
	if (this.sharedVariables.Exists(name)) {
		this.sharedVariables.Item(name) = value;
	}
	else {
		this.sharedVariables.Add(name, value);
	}
}

Utils.prototype.getVariable = function(name)
{
	if (name == null) {
		return null;
	}
	if (! this.sharedVariables.Exists(name)) {
		return null;
	}
	return this.sharedVariables.Item(name);
}

Utils.prototype.createError = function(code, text)
{
	return new Error(code, text);
}

Utils.prototype.clearError = function()
{
	this.objError = null;
}

Utils.prototype.setError = function(obj)
{
	if (__isNull(this.objError)) {
		this.objError = obj;
	}
}

Utils.prototype.getError = function()
{
	return this.objError;
}

Utils.prototype.isErrorEmpty = function()
{
	return __isNull(this.objError);
}

Utils.prototype.clearPostAction = function()
{
	this.postAction = null;
}

Utils.prototype.setPostAction = function(action)
{
	this.postAction = action;
}

Utils.prototype.getPostAction = function()
{
	return __trim(__tostr(this.postAction));
}

Utils.prototype.setLogLevel = function(level)
{
	this.logLevel = level;
}

Utils.prototype.log = function(text, logLevel)
{
	if (logLevel > this.logLevel) {
		return;
	}
	if (typeof(WScript) != "undefined") {
		WScript.Echo(text);
	}
	if (this.logPath == null) {
		this.logPath = objFS.BuildPath(this.userDir, "ascon.log");
	}
	var ts = objFS.OpenTextFile(this.logPath, ForAppending, true);
	var prefix = (logLevel >= 0 && logLevel < LogPrefix.length) ? LogPrefix[logLevel] : "";
	ts.WriteLine(formatDate(new Date()) + " " + prefix + " " + text);
	ts.Close();
}

Utils.prototype.logError = function(text)
{
	this.log(text, Log_Error);
}

Utils.prototype.logTrace = function(text)
{
	this.log(text, Log_Trace);
}

Utils.prototype.logDebug = function(text)
{
	this.log(text, Log_Debug);
}

Utils.prototype.alert = function(msg)
{
	if (typeof(window) != "undefined") {
		window.alert(msg);
	}
}

Utils.prototype.getJScriptObjectName = function(obj)
{
	if (__isObject(obj)) {
		if ((typeof(obj.constructor) != "undefined") && obj.constructor != null) {
			if (__tostr(obj.constructor).match(/^\s*function +(\w+)/)) {
				return RegExp.$1;
			}
		}
	}
	return typeof(obj);
}

Utils.prototype.getJScriptProperties = function(obj)
{
	if (! __isObject(obj)) {
		return "";
	}
	try {
		var ary = [];
		for (var member in obj) {
			if (typeof(obj[member]) != "function") {
				ary.push(member);
			}
		}
		return ary.join(this.getSeparator());
	}
	catch (e) {
	}
	return "";
}

Utils.prototype.getJScriptMethods = function(obj)
{
	if (! __isObject(obj)) {
		return "";
	}
	try {
		var ary = [];
		for (var member in obj) {
			if (typeof(obj[member]) == "function") {
				ary.push(member+"()");
			}
		}
		return ary.join(this.getSeparator());
	}
	catch (e) {
	}
	return "";
}

Utils.prototype.println = function(text)
{
	consoleBuffer.println(text);
}

Utils.prototype.print = function(text)
{
	consoleBuffer.print(text);
}

Utils.prototype.printError = function(text)
{
	consoleBuffer.println("[ERROR] " + text);
}

Utils.prototype.flushConsole = function()
{
	updateConsole();
}

Utils.prototype.setLanguage = function(lang)
{
	this.language = lang.substr(0, 2);
	if (messageTable.Exists(this.language)) {
		this.messages = messageTable.Item(this.language);
	}
	if (this.language == "ja") {
		var scriptNames = scriptManager.getScriptNames();
		for (i in scriptNames) {
			var name = scriptNames[i];
			var sc = scriptManager.getScript(name).getScriptControl();
			try {
				sc.Run("ASConsole_setJPEnvironment")
			}
			catch (e) {}
		}
	}
}

Utils.prototype.getText = function(id)
{
	if (! this.messages.Exists(id)) {
		return null;
	}
	return this.messages.Item(id);
}

Utils.prototype.isRubyScriptEnabled = function()
{
	try {
		return this.validateRubyScript();
	}
	catch (e) {}
	return false;
}

Utils.prototype.comInfo = function(obj, member)
{
	try {
		this.validateRubyScript();
		this.scRubyScript.Run("ASConsole_comInfo", obj, member);
	}
	catch (e) {
		this.setError(e);
	}
}

Utils.prototype.comMethods = function(obj)
{
	try {
		this.validateRubyScript();
		this.scRubyScript.Run("ASConsole_comMethods", obj);
	}
	catch (e) {
		this.setError(e);
	}
}

Utils.prototype.comProperties = function(obj)
{
	try {
		this.validateRubyScript();
		this.scRubyScript.Run("ASConsole_comProperties", obj);
	}
	catch (e) {
		this.setError(e);
	}
}

Utils.prototype.validateRubyScript = function()
{
	try {
		if (this.scRubyScript == null) {
			this.scRubyScript = scriptManager.getScript(defaultRubyScript).getScriptControl();
		}
		if (this.scRubyScript != null) {
			return this.scRubyScript.Run("ASConsole_isRubyEnabled");
		}
	}
	catch(e) {}
	throw new Error(utils.getText("COM_FUNCTION_NEED_RUBY"));
}

Utils.prototype.loadText = function(filename, charset)
{
	var path = this.getAbsolutePath(filename);
	if (__isEmpty(charset)) {
		charset = "utf-8";
	}
	var stm = new ActiveXObject("ADODB.Stream");
	stm.Type = adTypeText;
	stm.Charset = charset;
	stm.Open();
	stm.LoadFromFile(path);
	var text = stm.ReadText();
	return text;
}

Utils.prototype.saveText = function(filename, text, charset)
{
	var path = this.getAbsolutePath(filename);
	if (__isEmpty(charset)) {
		charset = "utf-8";
	}
	var stm = new ActiveXObject("ADODB.Stream");
	stm.Type = adTypeText;
	stm.Charset = charset;
	stm.Open();
	stm.WriteText(text);
	stm.SaveToFile(path, adSaveCreateOverWrite);
	stm.Close();
}

Utils.prototype.load = function(path, charset)
{
	if (path.match(/^(https?:|file:)/)) {
		// ignore charset
		objXmlHttp.open('GET', path, false);
		try {
			objXmlHttp.send(null);
		}
		catch (e) {
			this.logError("load() failed.");
			this.logError(formatError(e));
			return null;
		}
		if (path.match(/file:/)) {
			// no support charset
			return objXmlHttp.responseText;
		}
		if (objXmlHttp.status != 200) {
			this.logError("load() failed.");
			this.logError(__formatstr("status code: {0}", objXmlHttp.status));
			return null;
		}
		if (__isEmpty(charset)) {
			return objXmlHttp.responseText;
		}
		return this.binaryToText(objXmlHttp.responseBody, charset);
	}
	return this.loadText(path, charset);
}

Utils.prototype.getAbsolutePath = function(path)
{
	if (path.match(/^[A-Za-z]:\\/)) {
		return path;
	}
	else if (path.match(/^\\\\/)) {
		return path;
	}
	return objFS.BuildPath(this.appDir, path);
}

Utils.prototype.binaryToText = function(buffer, charset)
{
	var stm = new ActiveXObject("ADODB.Stream");
	stm.Type = adTypeBinary;
	stm.Open();
	stm.Write(buffer);
	stm.Position = 0;
	stm.Type = adTypeText;
	stm.Charset = charset;
	return stm.ReadText();
}

var utils = new Utils();


///////////////////////////////////////////////////////////////////////
// Console buffer

function ConsoleBuffer(size)
{
	this.bufferSize = size;
	this.clear();
}

ConsoleBuffer.prototype.println = function(text)
{
	var s = this.currentLine;
	if (text != null) {
		s += __tostr(text);
	}
	// broken charactor replace to '_'
	s = s.replace(/[\x00-\x08\x0B\x0C\x0E-\x1F]+/g, "_");
	s = s.replace(/\r*\n/g, "\r\n");
	if (s.charAt(s.length - 1) != "\n") {
		s += "\r\n";
	}
	var removeText = this.lines[this.currentIndex];
	this.lines[this.currentIndex] = s;
	this.currentIndex = (this.currentIndex + 1) % this.lines.length;
	if (removeText != "") {
		this.buffer = this.buffer.substr(removeText.length);
	}
	this.buffer += s;
	this.currentLine = "";
}

ConsoleBuffer.prototype.print = function(text)
{
	if (__isEmpty(text)) {
		return;
	}
	var s = __tostr(text);
	this.currentLine += s;
	if (this.currentLine.charAt(this.currentLine.length-1) == "\n") {
		this.println();
	}
}

ConsoleBuffer.prototype.clear = function()
{
	this.buffer = "";
	this.lines = new Array(this.bufferSize);
	var i;
	for (i = 0; i < this.bufferSize; i++) {
		this.lines[i] = "";
	}
	this.currentLine = "";
	this.currentIndex = 0;
}

ConsoleBuffer.prototype.flush = function()
{
	if (this.currentLine != "") {
		this.println();
	}
}

ConsoleBuffer.prototype.getBuffer = function()
{
	return this.buffer;
}

var consoleBuffer = new ConsoleBuffer(consoleBufferSize);


///////////////////////////////////////////////////////////////////////

function HistoryBuffer(size)
{
	this.lines = new Array(size);
	this.multilines = new Array(size);
	var i;
	for (i = 0; i < size; i++) {
		this.lines[i] = "";
		this.multilines[i] = null;
	}
	this.used = 0;
	this.totalLines = 0;
	this.currentIndex = -1;
	this.lineCounter = 0;
}

HistoryBuffer.prototype.addText = function(text)
{
	if (__isEmpty(text)) {
		return;
	}
	text = this.getCorrectText(text);
	this.removeRepeatedItem(text);
	var i;
	for (i = this.lines.length - 2; i >= 0; i--) {
		this.lines[i + 1] = this.lines[i];
		this.multilines[i + 1] = this.multilines[i];
	}
	if (isMultiLineText(text)) {
		this.lines[0] = __formatstr("#multiline#{0}  ({1})",
				[__numstr(this.totalLines % 1000, 3), this.getTextHeader(text)]);
		this.multilines[0] = text;
	}
	else {
		this.lines[0] = text;
		this.multilines[0] = null;
	}
	if (this.used < this.lines.length) {
		this.used++;
	}
	this.totalLines++;
	this.resetIndex();
}

HistoryBuffer.prototype.getLabel = function(index)
{
	if (index == null) {
		index = this.currentIndex;
	}
	if (index < 0 || index >= this.used) {
		return "";
	}
	return this.lines[index];
}

HistoryBuffer.prototype.getText = function(index)
{
	if (index == null) {
		index = this.currentIndex;
	}
	if (index < 0 || index >= this.used) {
		return "";
	}
	if (this.multilines[index] != null) {
		return this.multilines[index];
	}
	else {
		return this.lines[index];
	}
}

HistoryBuffer.prototype.getCorrectText = function(text)
{
	var identifier = this.getMultilineIdentifier(text);
	if (identifier == null) {
		return text;
	}
	var i;
	for (i = 0; i < this.used; i++) {
		if (this.multilines[i] != null) {
			if (identifier == this.getMultilineIdentifier(this.lines[i])) {
				return this.multilines[i];
			}
		}
	}
	return text;
}

HistoryBuffer.prototype.getMultilineIdentifier = function(text)
{
	if (text.match(/^(#multiline#\d+)/)) {
		return RegExp.$1;
	}
	return null;
}

HistoryBuffer.prototype.removeRepeatedItem = function(text)
{
	var index = -1;
	var i;
	for (i = 0; i < this.used; i++) {
		if (this.multilines[i] != null) {
			if (text == this.multilines[i]) {
				index = i;
				break;
			}
		}
		else {
			if (text == this.lines[i]) {
				index = i;
				break;
			}
		}
	}
	if (index != -1) {
		for (; i < this.used - 1; i++) {
			this.lines[i] = this.lines[i + 1];
			this.multilines[i] = this.multilines[i + 1];
		}
		this.used--;
	}
}

HistoryBuffer.prototype.resetIndex = function()
{
	this.currentIndex = -1;
}

HistoryBuffer.prototype.nextIndex = function()
{
	if (this.currentIndex + 1 < this.used) {
		this.currentIndex++;
		return true;
	}
	return false;
}

HistoryBuffer.prototype.prevIndex = function()
{
	if (this.currentIndex > -1) {
		this.currentIndex--;
		return true;
	}
	return false;
}

HistoryBuffer.prototype.isMultiline = function(index)
{
	if (index < 0 || index >= this.used) {
		return false;
	}
	return (this.multilines[index] != null);
}

HistoryBuffer.prototype.getTextHeader = function(text)
{
	var s = text.replace(/\s+/g, " ");
	if (s.length > 40) {
		s = s.substr(0, 40) + " ...";
	}
	return s;
}

HistoryBuffer.prototype.getUsed = function()
{
	return this.used;
}

HistoryBuffer.prototype.dump = function()
{
	var i;
	for (i = this.used - 1; i >= 0; i--) {
		utils.println(__formatstr("{0}: {1}", [i+1, this.getLabel(i)]));
	}
}

HistoryBuffer.prototype.serialize = function()
{
	var s = "";
	for (var i = this.used - 1; i >= 0; i--) {
		var text = this.getText(i);
		text = text.replace(/\\/g, "\\\\");
		text = text.replace(/\r*\n/g, "\\n");
		text = text.replace(/\t/g, "\\t");
		text = text.replace(/"/g, "\\\"");
		s += __formatstr("historyBuffer.addText(\"{0}\");", [text]) + "\n";
	}
	return s;
}

var historyBuffer = new HistoryBuffer(historyBufferSize);


///////////////////////////////////////////////////////////////////////

function AliasManager()
{
	this.aliasTable = new ActiveXObject("Scripting.Dictionary");
}

AliasManager.prototype.add = function(keyword, text)
{
	if (this.aliasTable.Exists(keyword)) {
		this.aliasTable.Item(keyword) = text;
	}
	else {
		this.aliasTable.Add(keyword, text);
	}
}

AliasManager.prototype.get = function(keyword)
{
	if (! this.aliasTable.Exists(keyword)) {
		return null;
	}
	return this.aliasTable.Item(keyword);
}

AliasManager.prototype.remove = function(keyword, text)
{
	if (this.aliasTable.Exists(keyword)) {
		this.aliasTable.Remove(keyword);
	}
}

AliasManager.prototype.applyText = function(text)
{
	var s = __trim(text);
	if (s.match(/^([A-Za-z_]\w*)(?:|\s+.*)/m) == null) {
		return s;
	}
	var keyword = RegExp.$1;
	var replaceText = this.get(keyword);
	if (replaceText == null) {
		return s;
	}
	return s.replace(keyword, replaceText);
}

AliasManager.prototype.dump = function()
{
	var aliases = new VBArray(this.aliasTable.keys()).toArray().sort();
	for (i in aliases) {
		var s = aliases[i] + " = " + this.get(aliases[i]).replace(/\r*\n/g, " ") + "\n";
		utils.println(s);
	}
}

AliasManager.prototype.serialize = function()
{
	var s = "";
	var aliases = new VBArray(this.aliasTable.keys()).toArray().sort();
	for (i in aliases) {
		var keyword = aliases[i];
		var text = this.get(aliases[i]);
		text = text.replace(/\\/g, "\\\\");
		text = text.replace(/\r*\n/g, "\\n");
		text = text.replace(/\t/g, "\\t");
		text = text.replace(/"/g, "\\\"");
		s += __formatstr("aliasManager.add(\"{0}\", \"{1}\");", [keyword, text]) + "\n";
	}
	return s;
}

var aliasManager = new AliasManager();


///////////////////////////////////////////////////////////////////////

var userSetting = "";
function restoreUserSetting()
{
	try {
		var path = objFS.BuildPath(utils.getUserDir(), "ascon.dat");
		if (objFS.FileExists(path)) {
			var text = utils.loadText(path, "utf-8");
			eval(text);
			// please don't edit following line. system use string as special keyword.
			if (text.match(/=== custom user setting ===.*?\n((?:.|\n)*)/) != null) {
				userSetting = __trim(RegExp.$1);
			}
		}
	}
	catch (e) {
		utils.printError("restoreUserSetting() failed.");
		utils.printError(formatError(e));
		utils.logError("restoreUserSetting() failed.");
		utils.logError(formatError(e));
	}
}

function saveUserSetting()
{
	backupUserSetting();
	try {
		var path = objFS.BuildPath(utils.getUserDir(), "ascon.dat");
		var text = ""+
"//---------------------------\n"+
"// history of user operation\n"+
"//---------------------------\n";
		text += aliasManager.serialize();
		text += historyBuffer.serialize();
		var currentScriptName = currentScript.getName();
		text += __formatstr("changeScriptEngine(\"{0}\", true);", [currentScriptName])+"\n";
		// please don't edit following line. system use string as special keyword.
		text += "\n// === custom user setting ===\n";
		text += userSetting;
		text += "\n";
		utils.saveText(path, text, "utf-8");
	}
	catch (e) {
		utils.logError("saveUserSetting() failed.");
		utils.logError(formatError(e));
	}
}

function backupUserSetting()
{
	var basePath = objFS.BuildPath(utils.getUserDir(), "ascon.dat");
	var gen = backupFileNum;
	if (gen > 9) {
		gen = 9;
	}
	for (var i = gen - 1; i >= 0; i--) {
		var src = basePath + ((i == 0) ? "" : ("." + i));
		var dst = basePath + "." + (i + 1);
		if (objFS.FileExists(src)) {
			try {
				objFS.CopyFile(src, dst, true);
			}
			catch (e) {
				utils.logError("backupUserSetting() failed.");
				utils.logError(formatError(e));
			}
		}
	}
}


///////////////////////////////////////////////////////////////////////

function isMultiLineText(text)
{
	if (! __isString(text)) {
		return false;
	}
	return (text.indexOf("\n") != -1);
}

function formatError(e)
{
	var s = "";
	if (e == null) {
		s = "(unknown error)";
	}
	//if (e instanceof Error) {
	else if (! __isNull(e.number) && ! __isNull(e.description)) {
		s = "" + (e.number & 0xffff) + ": " + e.description;
	}
	else if (! __isNull(e.description)) {
		s = "" + e.description;
	}
	else {
		s = __tostr(e);
	}
	return s;
}

function formatDate(dt)
{
	return __formatstr("{0}/{1}/{2} {3}:{4}:{5}.{6}",[
		__numstr(dt.getFullYear(), 4),__numstr(dt.getMonth()+1, 2),__numstr(dt.getDate(), 2),
		__numstr(dt.getHours(), 2),__numstr(dt.getMinutes(), 2),__numstr(dt.getSeconds(), 2),
		__numstr(dt.getMilliseconds(), 3)]);
}
